/*  gmysql -- a graphical frontend to MySQL databases
    Copyright (C) 1998 Stephen R. Dodd <dirk@loth.demon.co.uk>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <gdk/gdkkeysyms.h>
#include "connectdlg.h"

static void connect_dlg_done( GtkWidget *widget, gpointer data );
static void connect_dlg_cancel( GtkWidget *widget, gpointer data );
static void connect_dlg_destroyed( GtkWidget *widget, gpointer data );
static gint connect_dlg_key_pressed( GtkWidget *widget, GdkEventKey *event );

ConnectDlg *connect_dlg_new( TableList *owner )
{
	ConnectDlg *connect_dlg;
	GtkWidget *table, *label, *entry, *button, *dialog;

	connect_dlg = g_new( ConnectDlg, 1 );
	connect_dlg->owner = owner;
	dialog = connect_dlg->dialog = gtk_dialog_new();
	gtk_signal_connect( GTK_OBJECT( dialog ), "destroy",
		GTK_SIGNAL_FUNC( connect_dlg_destroyed ), connect_dlg );
	gtk_signal_connect( GTK_OBJECT( dialog ), "key_press_event",
		GTK_SIGNAL_FUNC( connect_dlg_key_pressed ), NULL );

	table = gtk_table_new( 3, 2, FALSE );

	label = gtk_label_new( "Host" );
	gtk_label_set_justify( GTK_LABEL( label ), GTK_JUSTIFY_RIGHT );
	gtk_misc_set_alignment( GTK_MISC( label ), 1.0, 0.5 );
	entry = connect_dlg->host = gtk_entry_new();
	gtk_table_attach( GTK_TABLE( table ), label, 0, 1, 0, 1, GTK_FILL, 0, 5, 5 );
	gtk_table_attach( GTK_TABLE( table ), entry, 1, 2, 0, 1, GTK_FILL|GTK_EXPAND, 0, 5, 5 );
	gtk_widget_show( label ); gtk_widget_show( entry );

	label = gtk_label_new( "User" );
	gtk_label_set_justify( GTK_LABEL( label ), GTK_JUSTIFY_RIGHT );
	gtk_misc_set_alignment( GTK_MISC( label ), 1.0, 0.5 );
	entry = connect_dlg->user = gtk_entry_new();
	gtk_table_attach( GTK_TABLE( table ), label, 0, 1, 1, 2, GTK_FILL, 0, 5, 5 );
	gtk_table_attach( GTK_TABLE( table ), entry, 1, 2, 1, 2, GTK_FILL|GTK_EXPAND, 0, 5, 5 );
	gtk_widget_show( label ); gtk_widget_show( entry );

	label = gtk_label_new( "Password" );
	gtk_label_set_justify( GTK_LABEL( label ), GTK_JUSTIFY_RIGHT );
	gtk_misc_set_alignment( GTK_MISC( label ), 1.0, 0.5 );
	entry = connect_dlg->pw = gtk_entry_new();
	gtk_entry_set_visibility( GTK_ENTRY( entry ), FALSE );
	gtk_table_attach( GTK_TABLE( table ), label, 0, 1, 2, 3, GTK_FILL, 0, 5, 5 );
	gtk_table_attach( GTK_TABLE( table ), entry, 1, 2, 2, 3, GTK_FILL|GTK_EXPAND, 0, 5, 5 );
	gtk_widget_show( label ); gtk_widget_show( entry );

	gtk_box_pack_start( GTK_BOX( GTK_DIALOG( dialog )->vbox ), table, FALSE, TRUE, 0 );
	gtk_widget_show( table );
	gtk_widget_grab_focus( connect_dlg->host );	/* must come after table is shown? */

	gtk_box_set_homogeneous( GTK_BOX( GTK_DIALOG( dialog )->action_area ), FALSE );
	button = gtk_button_new_with_label( "Cancel" );
	gtk_signal_connect( GTK_OBJECT( button ), "clicked",
		GTK_SIGNAL_FUNC( connect_dlg_cancel ), connect_dlg );
	GTK_WIDGET_SET_FLAGS( button, GTK_CAN_DEFAULT );
	gtk_box_pack_end( GTK_BOX( GTK_DIALOG( dialog )->action_area ), button, FALSE, FALSE, 0 );
	gtk_widget_show( button );

	button = gtk_button_new_with_label( "Connect" );
	gtk_signal_connect( GTK_OBJECT( button ), "clicked",
	GTK_SIGNAL_FUNC( connect_dlg_done ), connect_dlg );
	GTK_WIDGET_SET_FLAGS( button, GTK_CAN_DEFAULT );
	gtk_box_pack_end( GTK_BOX( GTK_DIALOG( dialog )->action_area ), button, FALSE, FALSE, 0 );
	gtk_widget_grab_default( button );	/* must do after packing but before showing */
	gtk_widget_show( button );

	return connect_dlg;
}

void connect_dlg_show( ConnectDlg *connect_dlg )
{
	gtk_widget_show( connect_dlg->dialog );
}

void connect_dlg_done( GtkWidget *widget, gpointer data )
{
	ConnectDlg *connect_dlg;
	gchar *host, *user, *pw;

	connect_dlg = CONNECT_DLG(data);
	g_assert( connect_dlg );

	host = gtk_entry_get_text( GTK_ENTRY( connect_dlg->host ) );
	user = gtk_entry_get_text( GTK_ENTRY( connect_dlg->user ) );
	pw = gtk_entry_get_text( GTK_ENTRY( connect_dlg->pw ) );

	gtk_widget_hide( connect_dlg->dialog );
	while( gtk_events_pending()) gtk_main_iteration();
	table_list_connect( connect_dlg->owner, host, user, pw );
	gtk_widget_destroy( connect_dlg->dialog );
	
}

void connect_dlg_cancel( GtkWidget *widget, gpointer data )
{
	ConnectDlg *connect_dlg;

	connect_dlg = CONNECT_DLG(data);
	gtk_widget_destroy( connect_dlg->dialog );
}

gint connect_dlg_key_pressed( GtkWidget *widget, GdkEventKey *event )
{
	if( event->keyval == GDK_Escape )
		gtk_widget_destroy( widget );
}

void connect_dlg_destroyed( GtkWidget *widget, gpointer data )
{
	g_free( data );
}
