/*  gmysql -- a graphical frontend to MySQL databases
    Copyright (C) 1998 Stephen R. Dodd <dirk@loth.demon.co.uk>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <gdk/gdkkeysyms.h>
#include "idxfielddlg.h"

static void idx_field_dlg_done( GtkWidget *widget, gpointer data );
static void idx_field_dlg_cancel( GtkWidget *widget, gpointer data );
static void idx_field_dlg_destroyed( GtkWidget *widget, gpointer data );
static gint idx_field_dlg_key_pressed( GtkWidget *widget, GdkEventKey *event );
static void field_selected( GtkWidget *widget, gpointer data );

static gchar *selected_field;
static GSList *group;

IdxFieldDlg *idx_field_dlg_new( IndexDlg *owner )
{
	IdxFieldDlg *idx_field_dlg;
	GtkWidget *table, *label, *entry, *button, *dialog, *menu, *item;

	selected_field = NULL; group = NULL;
	
	idx_field_dlg = g_new( IdxFieldDlg, 1 );
	idx_field_dlg->owner = owner;
	dialog = idx_field_dlg->dialog = gtk_dialog_new();
	gtk_signal_connect( GTK_OBJECT( dialog ), "destroy",
		GTK_SIGNAL_FUNC( idx_field_dlg_destroyed ), idx_field_dlg );
	gtk_signal_connect( GTK_OBJECT( dialog ), "key_press_event",
		GTK_SIGNAL_FUNC( idx_field_dlg_key_pressed ), NULL );

	table = gtk_table_new( 1, 2, FALSE );

	label = gtk_label_new( "Add field" );
	gtk_label_set_justify( GTK_LABEL( label ), GTK_JUSTIFY_RIGHT );
	gtk_misc_set_alignment( GTK_MISC( label ), 1.0, 0.5 );
	entry = idx_field_dlg->field = gtk_option_menu_new();
	menu = idx_field_dlg->field_menu = gtk_menu_new();
	gtk_table_attach( GTK_TABLE( table ), label, 0, 1, 0, 1, GTK_FILL, 0, 5, 5 );
	gtk_table_attach( GTK_TABLE( table ), entry, 1, 2, 0, 1, GTK_FILL|GTK_EXPAND, 0, 5, 5 );
	gtk_widget_show( label ); gtk_widget_show( entry );

	gtk_box_pack_start( GTK_BOX( GTK_DIALOG( dialog )->vbox ), table, FALSE, TRUE, 0 );
	gtk_widget_show( table );
	gtk_widget_grab_focus( entry );	/* must come after table is shown? */

	gtk_box_set_homogeneous( GTK_BOX( GTK_DIALOG( dialog )->action_area ), FALSE );
	button = gtk_button_new_with_label( "Cancel" );
	gtk_signal_connect( GTK_OBJECT( button ), "clicked",
		GTK_SIGNAL_FUNC( idx_field_dlg_cancel ), idx_field_dlg );
	GTK_WIDGET_SET_FLAGS( button, GTK_CAN_DEFAULT );
	gtk_box_pack_end( GTK_BOX( GTK_DIALOG( dialog )->action_area ), button, FALSE, FALSE, 0 );
	gtk_widget_show( button );

	button = gtk_button_new_with_label( "Add" );
	gtk_signal_connect( GTK_OBJECT( button ), "clicked",
	GTK_SIGNAL_FUNC( idx_field_dlg_done ), idx_field_dlg );
	GTK_WIDGET_SET_FLAGS( button, GTK_CAN_DEFAULT );
	gtk_box_pack_end( GTK_BOX( GTK_DIALOG( dialog )->action_area ), button, FALSE, FALSE, 0 );
	gtk_widget_grab_default( button );	/* must do after packing but before showing */
	gtk_window_set_modal( GTK_WINDOW( dialog ), TRUE );
	gtk_widget_show( button );

	return idx_field_dlg;
}

void idx_field_dlg_show( IdxFieldDlg *idx_field_dlg )
{
	gtk_option_menu_set_menu( GTK_OPTION_MENU( idx_field_dlg->field ),
		idx_field_dlg->field_menu );
	gtk_widget_show( idx_field_dlg->dialog );
}

void idx_field_dlg_done( GtkWidget *widget, gpointer data )
{
	IdxFieldDlg *idx_field_dlg;
	gchar *member;

	idx_field_dlg = IDX_FIELD_DLG(data);
	g_assert( idx_field_dlg );

	gtk_widget_hide( idx_field_dlg->dialog );
	while( gtk_events_pending()) gtk_main_iteration();

	index_dlg_add_field( idx_field_dlg->owner, selected_field );

	gtk_widget_destroy( idx_field_dlg->dialog );
}

void idx_field_dlg_add_field( IdxFieldDlg *idx_field_dlg, gchar *field )
{
	GtkWidget *item;

	item = gtk_radio_menu_item_new_with_label( group, field );
	group = gtk_radio_menu_item_group( GTK_RADIO_MENU_ITEM( item ) );
	gtk_signal_connect( GTK_OBJECT( item ), "activate",
		GTK_SIGNAL_FUNC( field_selected ), field );
	if( !selected_field ) field_selected( NULL, field );
	gtk_widget_show( item );
	gtk_menu_append( GTK_MENU( idx_field_dlg->field_menu ), item );
}

void idx_field_dlg_cancel( GtkWidget *widget, gpointer data )
{
	IdxFieldDlg *idx_field_dlg;

	idx_field_dlg = IDX_FIELD_DLG(data);
	gtk_widget_destroy( idx_field_dlg->dialog );
}

gint idx_field_dlg_key_pressed( GtkWidget *widget, GdkEventKey *event )
{
	if( event->keyval == GDK_Escape )
		gtk_widget_destroy( widget );
}

void idx_field_dlg_destroyed( GtkWidget *widget, gpointer data )
{
	g_free( data );
}

void field_selected( GtkWidget *widget, gpointer data )
{
	selected_field = (gchar *)data;
}
