/*  gmysql -- a graphical frontend to MySQL databases
    Copyright (C) 1998 Stephen R. Dodd <dirk@loth.demon.co.uk>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <gdk/gdkkeysyms.h>
#include "setdlg.h"

static void set_dlg_done( GtkWidget *widget, gpointer data );
static void set_dlg_cancel( GtkWidget *widget, gpointer data );
static void set_dlg_destroyed( GtkWidget *widget, gpointer data );
static gint set_dlg_key_pressed( GtkWidget *widget, GdkEventKey *event );

SetDlg *set_dlg_new( FieldDlg *owner )
{
	SetDlg *set_dlg;
	GtkWidget *table, *label, *entry, *button, *dialog, *menu, *item;

	set_dlg = g_new( SetDlg, 1 );
	set_dlg->owner = owner;
	dialog = set_dlg->dialog = gtk_dialog_new();
	gtk_signal_connect( GTK_OBJECT( dialog ), "destroy",
		GTK_SIGNAL_FUNC( set_dlg_destroyed ), set_dlg );
	gtk_signal_connect( GTK_OBJECT( dialog ), "key_press_event",
		GTK_SIGNAL_FUNC( set_dlg_key_pressed ), NULL );

	table = gtk_table_new( 1, 2, FALSE );

	label = gtk_label_new( "Set member" );
	gtk_label_set_justify( GTK_LABEL( label ), GTK_JUSTIFY_RIGHT );
	gtk_misc_set_alignment( GTK_MISC( label ), 1.0, 0.5 );
	entry = set_dlg->member = gtk_entry_new();
	gtk_table_attach( GTK_TABLE( table ), label, 0, 1, 0, 1, GTK_FILL, 0, 5, 5 );
	gtk_table_attach( GTK_TABLE( table ), entry, 1, 2, 0, 1, GTK_FILL|GTK_EXPAND, 0, 5, 5 );
	gtk_widget_show( label ); gtk_widget_show( entry );

	gtk_box_pack_start( GTK_BOX( GTK_DIALOG( dialog )->vbox ), table, FALSE, TRUE, 0 );
	gtk_widget_show( table );
	gtk_widget_grab_focus( set_dlg->member );	/* must come after table is shown? */

	gtk_box_set_homogeneous( GTK_BOX( GTK_DIALOG( dialog )->action_area ), FALSE );
	button = gtk_button_new_with_label( "Cancel" );
	gtk_signal_connect( GTK_OBJECT( button ), "clicked",
		GTK_SIGNAL_FUNC( set_dlg_cancel ), set_dlg );
	GTK_WIDGET_SET_FLAGS( button, GTK_CAN_DEFAULT );
	gtk_box_pack_end( GTK_BOX( GTK_DIALOG( dialog )->action_area ), button, FALSE, FALSE, 0 );
	gtk_widget_show( button );

	button = gtk_button_new_with_label( "Add" );
	gtk_signal_connect( GTK_OBJECT( button ), "clicked",
	GTK_SIGNAL_FUNC( set_dlg_done ), set_dlg );
	GTK_WIDGET_SET_FLAGS( button, GTK_CAN_DEFAULT );
	gtk_box_pack_end( GTK_BOX( GTK_DIALOG( dialog )->action_area ), button, FALSE, FALSE, 0 );
	gtk_widget_grab_default( button );	/* must do after packing but before showing */
	gtk_widget_show( button );

	return set_dlg;
}

void set_dlg_show( SetDlg *set_dlg )
{
	gtk_widget_show( set_dlg->dialog );
}

void set_dlg_done( GtkWidget *widget, gpointer data )
{
	SetDlg *set_dlg;
	gchar *member;

	set_dlg = SET_DLG(data);
	g_assert( set_dlg );

	member = gtk_entry_get_text( GTK_ENTRY( set_dlg->member  ) );

	gtk_widget_hide( set_dlg->dialog );
	while( gtk_events_pending()) gtk_main_iteration();

	field_dlg_add_member( set_dlg->owner, member );

	gtk_widget_destroy( set_dlg->dialog );
	
}

void set_dlg_cancel( GtkWidget *widget, gpointer data )
{
	SetDlg *set_dlg;

	set_dlg = SET_DLG(data);
	gtk_widget_destroy( set_dlg->dialog );
}

gint set_dlg_key_pressed( GtkWidget *widget, GdkEventKey *event )
{
	if( event->keyval == GDK_Escape )
		gtk_widget_destroy( widget );
}

void set_dlg_destroyed( GtkWidget *widget, gpointer data )
{
	g_free( data );
}
